@@ -18,13 +18,15 @@
 // along with this program ; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 //
-// $Id: Federation.cc,v 3.80 2008/03/05 15:33:50 rousse Exp $
+// $Id: Federation.cc,v 3.95 2008/06/27 09:54:47 rousse Exp $
 // ----------------------------------------------------------------------------
 
 #include <config.h>
 #include "Federation.hh"
+#include "NM_Classes.hh"
 #include <sstream>
 #include <cassert>
+#include <memory>
 
 #ifdef _WIN32
 
@@ -41,6 +43,7 @@
 #include "ObjectClassAttribute.hh"
 #include "PrettyDebug.hh"
 #include "LBTS.hh"
+#include "NM_Classes.hh"
 
 #ifdef _WIN32
 #include <windows.h>
@@ -143,13 +146,11 @@ Federation::Federation(const char *federation_name,
     if ((federation_name == NULL) || (federation_handle == 0))
         throw RTIinternalError("Null init parameter in Federation creation.");
 
-    name = strdup(federation_name);
+    name = std::string(federation_name);
 
     // Default Attribute values
-    handle = federation_handle ;
-
-    FEDid = new char[strlen(FEDid_name)+1] ;
-    strcpy(FEDid,FEDid_name);
+    handle = federation_handle;
+    FEDid  = std::string(FEDid_name);    
 
     D.Out(pdInit, "New Federation created with Handle %d, now reading FOM.",
           handle);
@@ -162,7 +163,6 @@ Federation::Federation(const char *federation_name,
 
     cout << "New federation: " << name << endl ;
 
-
     // We should try to open FOM file from different
     // predefined places:
     //
@@ -173,7 +173,7 @@ Federation::Federation(const char *federation_name,
     // 3 - default (unix) installation place plus FEDid_name 
     //     "/usr/local/share/federation/" + FEDid_name
     //
-    string filename   = string(FEDid) ;
+    string filename   = FEDid;
     bool   filefound  = false;
     cout << "Looking for FOM file... " << endl ; 
 
@@ -220,23 +220,21 @@ Federation::Federation(const char *federation_name,
       throw CouldNotOpenFED("RTIG cannot find FED file.");
     }
 
-    // now really assign FEDid
-    free(FEDid);
-    FEDid = strdup(filename.c_str());
+    // now really assign FEDid   
+    FEDid = filename;
 
-    // Try to open to verify if file exists
-    FILE *fftry ;
-    if ( (fftry=fopen(FEDid,"r")) == NULL)
+    // Try to open to verify if file exists    
+    std::ifstream fedTry(FEDid.c_str());
+    if (!fedTry.is_open())
         {
         cout << "... failed : ";
         G.Out(pdGendoc,"exit Federation::Federation on exception CouldNotOpenFED");
         throw CouldNotOpenFED("RTIG have found but cannot open FED file");
         }
-    else
-        {
+    else {
         cout << "... opened." << endl ;
-        fclose(fftry) ;
-        }
+        fedTry.close();
+    }
 
     int  nbcar_filename = filename.length() ;
     bool is_a_fed       = false ;
@@ -267,11 +265,11 @@ Federation::Federation(const char *federation_name,
         throw CouldNotOpenFED("Incorrect FED file name : nor .fed nor .xml file");
     }
        
-    ifstream fdd(filename.c_str());
+    std::ifstream fedFile(filename.c_str());
 
-    if (fdd.is_open())
+    if (fedFile.is_open())
         {
-    	fdd.close();
+    	fedFile.close();
         if ( is_a_fed )
             {
 	    int err = fedparser::build(filename.c_str(), root, verbose);
@@ -283,14 +281,14 @@ Federation::Federation(const char *federation_name,
 
             // Retrieve the FED file last modification time(for Audit)
             STAT_STRUCT StatBuffer ;
-            #if defined(_WIN32) && not defined(__MINGW32__)
+            #if defined(_WIN32) && !defined(__MINGW32__)
             char MTimeBuffer[26];
             #else
             char *MTimeBuffer ;
             #endif
 
             if (STAT_FUNCTION(filename.c_str(), &StatBuffer) == 0) {
-            #if defined(_WIN32) && not defined(__MINGW32__)                
+            #if defined(_WIN32) && !defined(__MINGW32__)                
                 ctime_s(&MTimeBuffer[0],26,&StatBuffer.st_mtime);
             #else
                 MTimeBuffer = ctime(&StatBuffer.st_mtime);
@@ -345,11 +343,8 @@ Federation::~Federation()
 //     }
 //     clear();
 
-    // Free local allocations
-    free(name);
-    free(FEDid);
+    // Free local allocations    
     delete root ;
-
     delete server ;
 
 #ifdef FEDERATION_USES_MULTICAST
@@ -388,7 +383,7 @@ Federation::getHandle() const
 const char *
 Federation::getName() const
 {
-    return name ;
+    return name.c_str() ;
 }
 
 // ----------------------------------------------------------------------------
@@ -403,14 +398,13 @@ Federation::getNbRegulators() const
 //! Returns the FEDid name given in 'Create Federation Execution'.
 const char *
 Federation::getFEDid() const
-{
-    assert(FEDid != NULL );
-    return FEDid ;
+{    
+    return FEDid.c_str() ;
 }
 
 // ----------------------------------------------------------------------------
 /*! Add the Federate to the Federation, and return its new federate handle.
-  MAX_FEDERATE is the maximum number of federates per federation. Also send
+  Also send
   Null messages from all others federates to initialize its LBTS, and
   finally a RequestPause message if the Federation is already paused.
 */
@@ -438,39 +432,37 @@ Federation::add(const char *federate_name, SocketTCP *tcp_link)
     // Send, to the newly added federate, a Null message from each regulating
     // federate (i) with their logical time h(i). This permits to calculate
     // its LBTS.
-    NetworkMessage message ;
+    NM_Message_Null nullMessage;
+    NM_Announce_Synchronization_Point ASPMessage; 
     try {
 	std::vector<LBTS::FederateClock> v ;
 	regulators.get(v);
 
-        for (unsigned int i = 0 ; i < v.size(); ++i) {
-            message.type = NetworkMessage::MESSAGE_NULL ;
-            message.federation = handle ;
-	    message.federate = v[i].first ;
-	    message.date = v[i].second ;
-
+        for (unsigned int i = 0 ; i < v.size(); ++i) {        	
+            nullMessage.federation = handle ;
+	        nullMessage.federate = v[i].first ;
+	        nullMessage.setDate(v[i].second);
             D.Out(pdTerm,
 		  "Sending NULL message(type %d) from %d to new federate.",
-                  message.type, message.federate);
+                  nullMessage.getType(), nullMessage.federate);
 
-            message.write(tcp_link);
+            nullMessage.send(tcp_link,NM_msgBufSend);
         }
 
         // If federation is synchronizing, put federate in same state.
-        if (isSynchronizing()) {
-            message.type = NetworkMessage::ANNOUNCE_SYNCHRONIZATION_POINT ;
-            message.federate = federate_handle ;
-            message.federation = handle ;
+        if (isSynchronizing()) {           
+            ASPMessage.federate = federate_handle ;
+            ASPMessage.federation = handle ;
 
             std::map<const char *, const char *>::const_iterator i ;
             i = synchronizationLabels.begin();
             for (; i != synchronizationLabels.end(); i++) {
-                message.setLabel((*i).first);
-                message.setTag((*i).second);
+                ASPMessage.setLabel((*i).first);
+                ASPMessage.setTag((*i).second);
                 D.Out(pdTerm, "Sending synchronization message %s (type %d)"
-                      " to the new Federate.", (*i).first, message.type);
+                      " to the new Federate.", (*i).first, ASPMessage.getType());
 
-                message.write(tcp_link);
+                ASPMessage.send(tcp_link,NM_msgBufSend);
                 federates.back().addSynchronizationLabel((*i).first);
             }
         }
@@ -528,13 +520,12 @@ Federation::addRegulator(FederateHandle federate_handle, FederationTime time)
     D.Out(pdTerm, "Federation %d: Federate %d is now a regulator(Time=%f).",
           handle, federate_handle, time);
 
-    NetworkMessage msg ;
-    msg.type = NetworkMessage::SET_TIME_REGULATING ;
+    NM_Set_Time_Regulating msg ;
     msg.exception = e_NO_EXCEPTION ;
     msg.federation = handle ;
     msg.federate = federate_handle ;
-    msg.regulator = true ;
-    msg.date = time ;
+    msg.regulatorOn();
+    msg.setDate(time);
 
     this->broadcastAnyMessage(&msg, 0);
 }
@@ -557,7 +548,7 @@ Federation::broadcastAnyMessage(NetworkMessage *msg,
 #else
                 socket = server->getSocketLink(i->getHandle());
 #endif
-                msg->write(socket);
+                msg->send(socket,NM_msgBufSend);
             }
             catch (RTIinternalError &e) {
                 D[pdExcept] << "Reference to a killed Federate while "
@@ -577,13 +568,13 @@ Federation::broadcastAnyMessage(NetworkMessage *msg,
 void
 Federation::broadcastSomeMessage(NetworkMessage *msg,
                                 FederateHandle except_federate,
-                                FederateHandle *fede_array,
+                                std::vector <FederateHandle> &fede_array,
                                 int nbfed)
 {
     int ifed ;
     Socket *socket = NULL ;
 
-    if ( fede_array != NULL || nbfed == 0)
+    if ( fede_array.size() != 0 || nbfed == 0)
         {
         // Broadcast the message 'msg' to some Federates (done in fede_array)
         // in the Federation
@@ -605,7 +596,7 @@ Federation::broadcastSomeMessage(NetworkMessage *msg,
 #else
                             socket = server->getSocketLink(i->getHandle());
 #endif
-                            msg->write(socket);
+                            msg->send(socket,NM_msgBufSend);
                             }
                         catch (RTIinternalError &e)
                             {
@@ -631,8 +622,8 @@ Federation::broadcastSomeMessage(NetworkMessage *msg,
 void
 Federation::broadcastInteraction(FederateHandle federate_handle,
                                  InteractionClassHandle interaction,
-                                 ParameterHandle *parameter_handles,
-                                 ParameterLengthPair *parameter_values,
+                                 std::vector <ParameterHandle> &parameter_handles,
+                                 std::vector <ParameterLengthPair> &parameter_values,
                                  UShort list_size,
                                  FederationTime time,
 				 RegionHandle region_handle,
@@ -680,8 +671,8 @@ Federation::broadcastInteraction(FederateHandle federate_handle,
 void
 Federation::broadcastInteraction(FederateHandle federate_handle,
                                  InteractionClassHandle interaction,
-                                 ParameterHandle *parameter_handles,
-                                 ParameterLengthPair *parameter_values,
+                                 std::vector <ParameterHandle> &parameter_handles,
+                                 std::vector <ParameterLengthPair> &parameter_values,
                                  UShort list_size,
 				 RegionHandle region_handle,
                                  const char *tag)
@@ -795,8 +786,8 @@ Federation::registerSynchronization(FederateHandle federate,
 
     this->check(federate); // It may throw FederateNotExecutionMember.
 
-    if ((label == NULL) || (strlen(label) > MAX_USER_TAG_LENGTH))
-        throw RTIinternalError("Bad pause label(null or too long).");
+    if (label == NULL )
+        throw RTIinternalError("Bad pause label(null).");
 
     // Verify label does not already exists
     std::map<const char *, const char *>::const_iterator i ;
@@ -832,7 +823,7 @@ Federation::registerSynchronization(FederateHandle federate,
                                     const char *label,
                                     const char *tag,
                                     unsigned short federate_setSize,
-                                    FederateHandle *federate_set)
+                                    std::vector <FederateHandle> &federate_set)
     throw (FederateNotExecutionMember,
            FederationAlreadyPaused,
            SaveInProgress,
@@ -844,8 +835,8 @@ Federation::registerSynchronization(FederateHandle federate,
 
     this->check(federate); // It may throw FederateNotExecutionMember.
 
-    if ((label == NULL) || (strlen(label) > MAX_USER_TAG_LENGTH))
-        throw RTIinternalError("Bad pause label(null or too long).");
+    if (label == NULL)
+        throw RTIinternalError("Bad pause label(null).");
 
     // Verify label does not already exists
     std::map<const char *, const char *>::const_iterator i ;
@@ -894,12 +885,11 @@ Federation::broadcastSynchronization(FederateHandle federate,
 
     this->check(federate); // It may throw FederateNotExecutionMember.
 
-    if ((label == NULL) || (strlen(label) > MAX_USER_TAG_LENGTH))
-        throw RTIinternalError("Bad pause label(null or too long).");
+    if (label == NULL)
+        throw RTIinternalError("Bad pause label(null).");
 
     // broadcast announceSynchronizationPoint() to all federates in federation.
-    NetworkMessage msg ;
-    msg.type = NetworkMessage::ANNOUNCE_SYNCHRONIZATION_POINT ;
+    NM_Announce_Synchronization_Point msg ;    
     msg.federate = federate ;
     msg.federation = handle ;
     msg.setLabel(label);
@@ -922,7 +912,7 @@ Federation::broadcastSynchronization(FederateHandle federate,
                                      const char *label,
                                      const char *tag,
                                      unsigned short federate_setSize,
-                                     FederateHandle *federate_set)
+                                     std::vector <FederateHandle> &federate_set)
     throw (RTIinternalError)
 {
 
@@ -930,12 +920,11 @@ Federation::broadcastSynchronization(FederateHandle federate,
 
     this->check(federate); // It may throw FederateNotExecutionMember.
 
-    if ((label == NULL) || (strlen(label) > MAX_USER_TAG_LENGTH))
+    if (label == NULL)
         throw RTIinternalError("Bad pause label(null or too long).");
 
     // broadcast announceSynchronizationPoint() to all federates in federation.
-    NetworkMessage msg ;
-    msg.type = NetworkMessage::ANNOUNCE_SYNCHRONIZATION_POINT ;
+    NM_Announce_Synchronization_Point msg ;    
     msg.federate = federate ;
     msg.federation = handle ;
     msg.setLabel(label);
@@ -976,14 +965,12 @@ Federation::requestFederationSave(FederateHandle the_federate,
     saveInProgress = true ;
     saveLabel = the_label ;
 
-    NetworkMessage msg ;
-    msg.type = NetworkMessage::INITIATE_FEDERATE_SAVE ;
+    NM_Initiate_Federate_Save msg ;    
     msg.federate = the_federate ;
     msg.federation = handle ;
     msg.setLabel(the_label);
-    // boolean true means with time and needs time
-    msg.setBoolean(true);
-    msg.date = time;
+    // timed message
+    msg.setDate(time);
 
     G.Out(pdGendoc,"      requestFederationSave====>broadcast I_F_S to all");
 
@@ -1017,13 +1004,10 @@ Federation::requestFederationSave(FederateHandle the_federate,
     saveInProgress = true ;
     saveLabel = the_label ;
 
-    NetworkMessage msg ;
-    msg.type = NetworkMessage::INITIATE_FEDERATE_SAVE ;
+    NM_Initiate_Federate_Save msg ;    
     msg.federate = the_federate ;
     msg.federation = handle ;
-    msg.setLabel(the_label);
-    // boolean false means without time
-    msg.setBoolean(false);
+    msg.setLabel(the_label);    
 
     G.Out(pdGendoc,"                  requestFederationSave====>broadcast I_F_S"
                    " to all");
@@ -1075,14 +1059,12 @@ Federation::federateSaveStatus(FederateHandle the_federate, bool the_status)
     }
 
     // Send end save message.
-    NetworkMessage msg ;
+    std::auto_ptr<NetworkMessage> msg(NM_Factory::create(saveStatus ? NetworkMessage::FEDERATION_SAVED : NetworkMessage::FEDERATION_NOT_SAVED )) ;
 
-    msg.type = saveStatus ? NetworkMessage::FEDERATION_SAVED : NetworkMessage::FEDERATION_NOT_SAVED ;
+    msg->federate = the_federate ;
+    msg->federation = handle ;
 
-    msg.federate = the_federate ;
-    msg.federation = handle ;
-
-    broadcastAnyMessage(&msg, 0);
+    broadcastAnyMessage(msg.get(), 0);
 
     G.Out(pdGendoc,"            =======> broadcast F_S or F_N_S");
 
@@ -1108,11 +1090,8 @@ Federation::requestFederationRestore(FederateHandle the_federate,
         throw RestoreInProgress("Already in restoring state.");
 
     Socket * socket ;
-
-    NetworkMessage * msg = new NetworkMessage ;
-    msg->federate = the_federate ;
-    msg->federation = handle ;
-    msg->setLabel(the_label);
+    NetworkMessage *msg;
+    
 
     // Informs sending federate of success/failure in restoring.
     // At this point, only verify that file is present.
@@ -1137,9 +1116,15 @@ Federation::requestFederationRestore(FederateHandle the_federate,
 // JYR Note : forcing success to true to skip xmlParseFile (not compliant ?)
     success = true ;
 
-    msg->type = success ?
-        NetworkMessage::REQUEST_FEDERATION_RESTORE_SUCCEEDED
-        : NetworkMessage::REQUEST_FEDERATION_RESTORE_FAILED ;
+    if (success) {
+    	msg = NM_Factory::create(NetworkMessage::REQUEST_FEDERATION_RESTORE_SUCCEEDED);
+    } else {
+    	msg = NM_Factory::create(NetworkMessage::REQUEST_FEDERATION_RESTORE_FAILED);
+    }
+    
+    msg->federate = the_federate ;
+    msg->federation = handle ;
+    msg->setLabel(the_label);
 
     socket = server->getSocketLink(msg->federate);
 
@@ -1148,7 +1133,7 @@ Federation::requestFederationRestore(FederateHandle the_federate,
     else
        G.Out(pdGendoc,"             =====> send message R_F_R_F to RTIA");
 
-    msg->write(socket);
+    msg->send(socket,NM_msgBufSend);
     delete msg ;
 
     // Reading file failed: not restoring !
@@ -1166,10 +1151,9 @@ Federation::requestFederationRestore(FederateHandle the_federate,
     restoreInProgress = true ;
 
     // Informs federates a new restore is being done.
-    msg = new NetworkMessage ;
+    msg = NM_Factory::create(NetworkMessage::FEDERATION_RESTORE_BEGUN);
     msg->federate = the_federate ;
-    msg->federation = handle ;
-    msg->type = NetworkMessage::FEDERATION_RESTORE_BEGUN ;
+    msg->federation = handle ;    
 
     G.Out(pdGendoc,"             =====> broadcast message F_R_B");
 
@@ -1177,10 +1161,9 @@ Federation::requestFederationRestore(FederateHandle the_federate,
     delete msg ;
 
     // For each federate, send an initiateFederateRestore with correct handle.
-    msg = new NetworkMessage ;
+    msg = NM_Factory::create(NetworkMessage::INITIATE_FEDERATE_RESTORE);
     msg->federation = handle ;
-    msg->setLabel(the_label);
-    msg->type = NetworkMessage::INITIATE_FEDERATE_RESTORE ;
+    msg->setLabel(the_label);   
 
     for (FederateList::iterator i = federates.begin(); i != federates.end(); ++i) {
         msg->federate = i->getHandle();
@@ -1188,8 +1171,9 @@ Federation::requestFederationRestore(FederateHandle the_federate,
         // send message.
         socket = server->getSocketLink(msg->federate);
         G.Out(pdGendoc,"             =====> send message I_F_R to federate %d",msg->federate);
-        msg->write(socket);
+        msg->send(socket,NM_msgBufSend);
     }
+    delete msg;
     G.Out(pdGendoc,"exit  Federation::requestFederationRestore");
 }
 
@@ -1217,17 +1201,12 @@ Federation::federateRestoreStatus(FederateHandle the_federate,
     }
 
     // Send end restore message.
-    NetworkMessage msg ;
+    std::auto_ptr<NetworkMessage> msg(NM_Factory::create(restoreStatus ? NetworkMessage::FEDERATION_RESTORED : NetworkMessage::FEDERATION_NOT_RESTORED)) ;
+        
+    msg->federate = the_federate ;
+    msg->federation = handle ;
 
-    if (restoreStatus)
-        msg.type = NetworkMessage::FEDERATION_RESTORED ;
-    else
-        msg.type = NetworkMessage::FEDERATION_NOT_RESTORED ;
-
-    msg.federate = the_federate ;
-    msg.federation = handle ;
-
-    broadcastAnyMessage(&msg, 0);
+    broadcastAnyMessage(msg.get(), 0);
 
     // Reinitialize state.
     restoreStatus = true ;
@@ -1365,7 +1344,7 @@ Federation::publishInteraction(FederateHandle federate,
 void
 Federation::publishObject(FederateHandle federate,
                           ObjectClassHandle object,
-                          AttributeHandle *attributes,
+                          std::vector <AttributeHandle> &attributes,
                           UShort list_size,
                           bool pub)
     throw (ObjectClassNotDefined,
@@ -1492,13 +1471,10 @@ Federation::removeRegulator(FederateHandle federate_handle)
     D.Out(pdTerm, "Federation %d: Federate %d is not a regulator anymore.",
           handle, federate_handle);
 
-    NetworkMessage msg ;
-    msg.type = NetworkMessage::SET_TIME_REGULATING ;
-    msg.exception = e_NO_EXCEPTION ;
+    NM_Set_Time_Regulating msg ;    
     msg.federation = handle ;
     msg.federate = federate_handle ;
-    msg.regulator = false ;
-    msg.date = 0 ;
+    msg.regulatorOff();    
 
     broadcastAnyMessage(&msg, 0);
 }
@@ -1519,8 +1495,8 @@ Federation::unregisterSynchronization(FederateHandle federate_handle,
 
     this->check(federate_handle); // It may throw FederateNotExecutionMember.
 
-    if ((label == NULL) || (strlen(label) > MAX_USER_TAG_LENGTH))
-        throw RTIinternalError("Bad pause label(null or too long).");
+    if (label == NULL)
+        throw RTIinternalError("Bad pause label(null).");
 
     // Set federate synchronized on this label.
     Federate &federate = getFederate(federate_handle);
@@ -1549,9 +1525,7 @@ Federation::unregisterSynchronization(FederateHandle federate_handle,
     }
 
     // send a federationSynchronized().
-    NetworkMessage msg ;
-    msg.type = NetworkMessage::FEDERATION_SYNCHRONIZED ;
-    msg.exception = e_NO_EXCEPTION ;
+    NM_Federation_Synchronized msg ;    
     msg.federation = handle ;
     msg.federate = federate_handle ;
     msg.setLabel(label);
@@ -1594,7 +1568,7 @@ Federation::subscribeInteraction(FederateHandle federate,
 void
 Federation::subscribeObject(FederateHandle federate,
                             ObjectClassHandle object,
-                            AttributeHandle *attributes,
+                            std::vector <AttributeHandle> &attributes,
                             UShort list_size)
     throw (ObjectClassNotDefined,
            AttributeNotDefined,
@@ -1622,8 +1596,8 @@ Federation::subscribeObject(FederateHandle federate,
 void
 Federation::updateAttributeValues(FederateHandle federate,
                                   ObjectHandle id,
-                                  AttributeHandle *attributes,
-                                  ValueLengthPair *values,
+                                  std::vector <AttributeHandle> &attributes,
+                                  std::vector <ValueLengthPair> &values,
                                   UShort list_size,
                                   FederationTime time,
                                   const char *tag)
@@ -1635,6 +1609,7 @@ Federation::updateAttributeValues(FederateHandle federate,
            RestoreInProgress,
            RTIinternalError)
 {
+    G.Out(pdGendoc,"enter Federation::updateAttributeValues with time");
     // It may throw FederateNotExecutionMember.
     this->check(federate);
 
@@ -1645,6 +1620,7 @@ Federation::updateAttributeValues(FederateHandle federate,
     D.Out(pdRegister,
           "Federation %d: Federate %d updated attributes of Object %d.",
           handle, federate, id);
+    G.Out(pdGendoc,"exit  Federation::updateAttributeValues with time");
 }
 
 // ----------------------------------------------------------------------------
@@ -1653,8 +1629,8 @@ Federation::updateAttributeValues(FederateHandle federate,
 void
 Federation::updateAttributeValues(FederateHandle federate,
                                   ObjectHandle id,
-                                  AttributeHandle *attributes,
-                                  ValueLengthPair *values,
+                                  std::vector <AttributeHandle> &attributes,
+                                  std::vector <ValueLengthPair> &values,
                                   UShort list_size,
                                   const char *tag)
     throw (FederateNotExecutionMember,
@@ -1665,6 +1641,7 @@ Federation::updateAttributeValues(FederateHandle federate,
            RestoreInProgress,
            RTIinternalError)
 {
+    G.Out(pdGendoc,"enter Federation::updateAttributeValues without time");
     // It may throw FederateNotExecutionMember.
     this->check(federate);
 
@@ -1675,6 +1652,7 @@ Federation::updateAttributeValues(FederateHandle federate,
     D.Out(pdRegister,
           "Federation %d: Federate %d updated attributes of Object %d.",
           handle, federate, id);
+    G.Out(pdGendoc,"exit  Federation::updateAttributeValues without time");
 }
 // ----------------------------------------------------------------------------
 //! Update the current time of a regulator federate.
@@ -1697,12 +1675,10 @@ Federation::updateRegulator(FederateHandle federate_handle,
 
     regulators.update(federate_handle, time);
 
-    NetworkMessage msg ;
-    msg.type = NetworkMessage::MESSAGE_NULL ;
-    msg.exception = e_NO_EXCEPTION ;
+    NM_Message_Null msg ;   
     msg.federation = handle ;
     msg.federate = federate_handle ;
-    msg.date = time ;
+    msg.setDate(time);
 
     broadcastAnyMessage(&msg, federate_handle);
 }
@@ -1758,7 +1734,7 @@ Federation::queryAttributeOwnership(FederateHandle federate,
 void
 Federation::negotiateDivestiture(FederateHandle federate,
                                  ObjectHandle id,
-                                 AttributeHandle *attribs,
+                                 std::vector <AttributeHandle> &attribs,
                                  UShort list_size,
                                  const char *tag)
     throw (FederateNotExecutionMember,
@@ -1787,7 +1763,7 @@ Federation::negotiateDivestiture(FederateHandle federate,
 void
 Federation::acquireIfAvailable(FederateHandle federate,
                                ObjectHandle id,
-                               AttributeHandle *attribs,
+                               std::vector <AttributeHandle> &attribs,
                                UShort list_size)
     throw (ObjectNotKnown,
            ObjectClassNotPublished,
@@ -1816,7 +1792,7 @@ Federation::acquireIfAvailable(FederateHandle federate,
 void
 Federation::divest(FederateHandle federate,
                    ObjectHandle id,
-                   AttributeHandle *attrs,
+                   std::vector <AttributeHandle> &attrs,
                    UShort list_size)
     throw (ObjectNotKnown,
            AttributeNotDefined,
@@ -1842,7 +1818,7 @@ Federation::divest(FederateHandle federate,
 void
 Federation::acquire(FederateHandle federate,
                     ObjectHandle id,
-                    AttributeHandle *attributes,
+                    std::vector <AttributeHandle> &attributes,
                     UShort list_size,
                     const char *tag)
     throw (ObjectNotKnown,
@@ -1874,7 +1850,7 @@ Federation::acquire(FederateHandle federate,
 void
 Federation::cancelDivestiture(FederateHandle federate,
                               ObjectHandle id,
-                              AttributeHandle *attributes,
+                              std::vector <AttributeHandle> &attributes,
                               UShort list_size)
     throw (ObjectNotKnown,
            AttributeNotDefined,
@@ -1903,7 +1879,7 @@ Federation::cancelDivestiture(FederateHandle federate,
 AttributeHandleSet*
 Federation::respondRelease(FederateHandle federate,
                            ObjectHandle id,
-                           AttributeHandle *attributes,
+                           std::vector <AttributeHandle> &attributes,
                            UShort list_size)
     throw (ObjectNotKnown,
            AttributeNotDefined,
@@ -1932,7 +1908,7 @@ Federation::respondRelease(FederateHandle federate,
 void
 Federation::cancelAcquisition(FederateHandle federate,
                               ObjectHandle id,
-                              AttributeHandle *attributes,
+                              std::vector <AttributeHandle> &attributes,
                               UShort list_size)
     throw (ObjectNotKnown,
            AttributeNotDefined,
@@ -2008,7 +1984,7 @@ Federation::associateRegion(FederateHandle federate,
 			    ObjectHandle object,
 			    RegionHandle the_handle,
 			    unsigned short nb,
-			    AttributeHandle *attributes)
+			    std::vector <AttributeHandle> &attributes)
     throw (RegionNotKnown, SaveInProgress, RestoreInProgress, RTIinternalError)
 {
     check(federate);
@@ -2043,7 +2019,7 @@ Federation::subscribeAttributesWR(FederateHandle federate,
 				  ObjectClassHandle c,
 				  RegionHandle region_handle,
 				  unsigned short nb,
-				  AttributeHandle *attributes)
+				  std::vector <AttributeHandle> &attributes)
     throw (RegionNotKnown,
 	   SaveInProgress,
 	   RestoreInProgress,
@@ -2113,7 +2089,7 @@ Federation::registerObjectWithRegion(FederateHandle federate,
 				     const char *object_name,
 				     RegionHandle region_handle,
 				     int nb,
-				     AttributeHandle *attributes)
+				     std::vector <AttributeHandle> &attributes)
     throw (ObjectClassNotDefined, ObjectClassNotPublished,
 	   AttributeNotDefined, AttributeNotPublished, RegionNotKnown,
 	   InvalidRegionContext, ObjectAlreadyRegistered,
@@ -2179,7 +2155,7 @@ Federation::restoreXmlData()
         return false ;
     }
 
-    if (strcmp(name, XmlParser::CleanXmlGetProp(cur,(const xmlChar*)"name")) != 0) {
+    if (strcmp(name.c_str(), XmlParser::CleanXmlGetProp(cur,(const xmlChar*)"name")) != 0) {
         cerr << "Wrong federation name" << endl ;
     }
     
@@ -2231,7 +2207,7 @@ Federation::saveXmlData()
     xmlNodePtr federation ;
     federation = xmlNewChild(doc->children, NULL, NODE_FEDERATION, NULL);
 
-    xmlSetProp(federation, (const xmlChar *) "name", (const xmlChar *) name);
+    xmlSetProp(federation, (const xmlChar *) "name", (const xmlChar *) name.c_str());
 
     char t[10] ;
     sprintf(t, "%ld", handle);
@@ -2270,31 +2246,40 @@ Federation::saveXmlData()
 FederateHandle
 Federation::requestObjectOwner(FederateHandle theFederateHandle,
                              ObjectHandle theObject,
-                             AttributeHandle *theAttributeList,
+                             std::vector <AttributeHandle> &theAttributeList,
                              UShort theListSize)
         throw (ObjectNotKnown)
 {
 FederateHandle theOwnerHandle ;
-NetworkMessage mess ;
+NM_Provide_Attribute_Value_Update mess ;
 
     G.Out(pdGendoc,"enter Federation::requestObjectOwner");
 
     // Request Object.
     theOwnerHandle = root->requestObjectOwner(theFederateHandle,theObject) ;
 
-    // Send a PROVIDE_ATTRIBUTE_VALUE_UPDATE to the owner
-    mess.type = NetworkMessage::PROVIDE_ATTRIBUTE_VALUE_UPDATE ;
+    // Send a PROVIDE_ATTRIBUTE_VALUE_UPDATE to the owner    
     mess.federate = theFederateHandle ;
     mess.object = theObject ;
     mess.handleArraySize = theListSize ;
+    mess.handleArray.resize(theListSize) ;
     for (int i = 0 ; i < theListSize ; i++)
         {
         mess.handleArray[i] = theAttributeList[i] ;
         }
 
-     mess.write(server->getSocketLink(theOwnerHandle));
+     // JYR : BUG if getSocketLink return NULL means
+     // owner federate has been killed and so rtig don't crash
+     // better development needed
+     if ( server->getSocketLink(theOwnerHandle) == NULL )
+        {
+        throw ObjectNotKnown ( "Owner federate killed") ;
+        }
+
+     mess.send(server->getSocketLink(theOwnerHandle),NM_msgBufSend);
  
-   
+   G.Out(pdGendoc,"            requestObjectOwner ===> write PAVU to RTIA %d"
+                   ,theOwnerHandle);
     G.Out(pdGendoc,"exit  Federation::requestObjectOwner");
     return(theOwnerHandle);
 
@@ -2302,5 +2287,5 @@ NetworkMessage mess ;
 
 }} // namespace certi/rtig
 
-// $Id: Federation.cc,v 3.80 2008/03/05 15:33:50 rousse Exp $
+// $Id: Federation.cc,v 3.95 2008/06/27 09:54:47 rousse Exp $
 
